module Fss
  VERSION = '0.0.1'.freeze
  # Controller concern for handling commands
  module Commandable
    include ActiveSupport::Concern
    include Dry::Monads::Result::Mixin

    def command(command_params)
      @command_params = command_params
      yield
    end

    def form(form_class)
      execute_form form_class
    end

    def service(service_class, *params)
      execute_service service_class, command_params, *params if validation_result.success?
    end

    def serializer(serializer_class)
      execute_serialization serializer_class, service_result.value! if service_result.success?
    end

    private

    def command_params
      @command_params
    end

    def execute_form(form_class)
      result = form_class.validate(command_params) ? Success(command_params) : Failure(form_class.errors)
      @validation_result = result
    end

    def validation_result
      @validation_result
    end

    def execute_service(service_class, *params)
      result = service_class.call(*params)
      @service_result = result
    end

    def service_result
      @service_result
    end

    def execute_serialization(serializer_class, params)
      Success(serializer_class.new(params))
    end
  end
end
